Package: hydroTSM
Type: Package
Title: Time Series Management and Analysis for Hydrological Modelling
Version: 0.8-6
Authors@R: c( person("Mauricio Zambrano-Bigiarini", email = "mzb.devel@gmail.com", role=c("aut", "cre", "cph"), comment=c(ORCID = "0000-0002-9536-643X")), person("Joschka", "Thurner", email="git@joschkathurner.de", role=c("ctb")) )
Maintainer: Mauricio Zambrano-Bigiarini <mzb.devel@gmail.com>
Description: S3 functions for management, analysis, interpolation and plotting of time series used in hydrology and related environmental sciences. In particular, this package is highly oriented to hydrological modelling tasks. The focus of this package has been put in providing a collection of tools useful for the daily work of hydrologists (although an effort was made to optimise each function as much as possible, functionality has had priority over speed). Bugs / comments / questions / collaboration of any kind are very welcomed, and in particular, datasets that can be included in this package for academic purposes.
License: GPL (>= 2)
Depends: R (>= 3.5.0), zoo (>= 1.7-2)
Imports: xts (>= 0.9-7), e1071, stats, utils, methods, graphics,
        grDevices, lattice, classInt, timechange
Suggests: knitr, rmarkdown, gridExtra
VignetteBuilder: knitr
URL: https://hzambran.github.io/hydroTSM/,
        https://github.com/hzambran/hydroTSM,
        https://CRAN.R-project.org/package=hydroTSM
MailingList: https://stat.ethz.ch/mailman/listinfo/r-sig-ecology
BugReports: https://github.com/hzambran/hydroTSM/issues
LazyLoad: yes
NeedsCompilation: no
Repository: CRAN
Packaged: 2026-04-28 01:34:48 UTC; hzambran
Author: Mauricio Zambrano-Bigiarini [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9536-643X>),
  Joschka Thurner [ctb]
Date/Publication: 2026-04-28 07:20:03 UTC
Built: R 4.6.0; ; 2026-04-28 11:14:03 UTC; unix
