importFrom("stats", time, cor, cor.test, symnum, plot.stepfun, ecdf, median, splinefun, integrate, filter, IQR, sd, start, end, window, aggregate, approxfun, fivenum, lm, quantile, rnorm, runif)
importFrom("graphics", boxplot, plot, hist, par, layout, axis, Axis, abline, lines, points, legend, box, text, strwidth, rect, polygon, pairs, panel.smooth)
importFrom("grDevices", xy.coords, palette, colorRampPalette, dev.set, dev.list, dev.new, dev.off, png, rainbow) 
importFrom("utils", read.table, write.table, write.csv, write.csv2, installed.packages, ls.str, modifyList, sessionInfo, setTxtProgressBar, txtProgressBar) 
importFrom("methods", as, hasArg, is, slot) 
importFrom("graphics", barplot, grid, axTicks, Axis, abline, axis, boxplot, hist, legend, lines, mtext, par, plot)
importFrom("classInt", classIntervals)

importFrom("zoo", is.zoo, as.Date, as.zoo, zoo, coredata, as.yearmon, rollapply, index, "time<-", na.approx, na.spline)
importFrom("xts", axTicksByTime, to.period, .index, periodicity)
#importFrom("zoo", aggregate) # aggregate is not exported from zoo
importFrom("timechange", time_force_tz)
importFrom("e1071", skewness, kurtosis)
importFrom("lattice", levelplot)
#importFrom("lattice", lattice.options, lattice.getOption, trellis.currentLayout, trellis.focus, trellis.unfocus, trellis.panelArgs)
#importFrom("grid", grid.lines, gpar)

#importFrom("maptools", readShapePoly)
#importFrom("gstat", krige, idw)
#importFrom("automap", autoKrige)
#importFrom("sp", proj4string, coordinates, "coordinates<-", "proj4string<-", spplot, CRS, spsample, "gridded<-", SpatialPoints, layout.north.arrow, layout.scale.bar, over)

#importFrom("chron", month.day.year)
exportPattern("^[^\\.]")

export(shiftyears)

S3method(annualfunction, data.frame)
S3method(annualfunction, default)
S3method(annualfunction, matrix)
S3method(annualfunction, zoo)
S3method(baseflow, zoo)
S3method(calendarHeatmap, zoo)
S3method(cmv, default)
S3method(cmv, default)
S3method(cmv, zoo) 
S3method(cmv, data.frame) 
S3method(cmv, matrix) 
S3method(daily2annual, default)
S3method(daily2annual, zoo)
S3method(daily2annual, data.frame)
S3method(daily2annual, matrix) 
S3method(daily2monthly, default)
S3method(daily2monthly, zoo)
S3method(daily2monthly, data.frame) 
S3method(daily2monthly, matrix) 
S3method(daily2weekly, default)
S3method(daily2weekly, zoo)
S3method(daily2weekly, data.frame) 
S3method(daily2weekly, matrix) 
S3method(dm2seasonal, default)
S3method(dm2seasonal, zoo)
S3method(dm2seasonal, data.frame) 
S3method(dm2seasonal, matrix) 
S3method(dwdays, default) 
#S3method(dwdays, zoo) 
S3method(dwdays, data.frame) 
S3method(dwdays, matrix)
S3method(dwi, default)  
S3method(dwi, zoo)
S3method(dwi, data.frame)
S3method(dwi, matrix) 
S3method(extract, default)
S3method(extract, zoo)
S3method(fdc, default) 
S3method(fdc, zoo) 
S3method(fdc, data.frame)
S3method(fdc, matrix) 
S3method(fdcu, default) 
#S3method(fdcu, zoo) 
S3method(fdcu, data.frame) 
S3method(fdcu, matrix)
#S3method(hydrokrige, data.frame) 
#S3method(hydrokrige, default) 
S3method(hydroplot, default)
S3method(hydroplot, zoo)
S3method(hydroplot, data.frame)
S3method(infillxy, data.frame)
S3method(infillxy, default) 
S3method(infillxy, matrix) 
S3method(istdx, data.frame) 
S3method(istdx, default)
S3method(istdx, matrix)
S3method(isComplete, zoo)
S3method(izoo2rzoo, default) 
S3method(izoo2rzoo, zoo) 
S3method(ma, default) 
S3method(ma, zoo)
S3method(monthlyfunction, default)
S3method(monthlyfunction, zoo)
S3method(monthlyfunction, data.frame)
S3method(monthlyfunction, matrix) 
S3method(plot_pq, zoo)
S3method(seasonalfunction, default) 
S3method(seasonalfunction, zoo) 
S3method(seasonalfunction, data.frame)
S3method(seasonalfunction, matrix) 
S3method(smry, Date)
S3method(smry, default) 
S3method(smry, zoo) 
S3method(smry, data.frame) 
S3method(smry, matrix) 
S3method(sname2plot, default)
S3method(sname2plot, zoo)
S3method(sname2plot, data.frame)
S3method(stdx, data.frame) 
S3method(stdx, default)
S3method(stdx, matrix) 
S3method(subdaily2daily, default)
S3method(subdaily2daily, zoo)
S3method(subdaily2daily, data.frame)
S3method(subdaily2daily, matrix)
S3method(subdaily2weekly, default)
S3method(subdaily2weekly, zoo)
S3method(subdaily2weekly, data.frame)
S3method(subdaily2weekly, matrix)
S3method(subdaily2monthly, default)
S3method(subdaily2monthly, zoo)
S3method(subdaily2monthly, data.frame)
S3method(subdaily2monthly, matrix)
S3method(subhourly2hourly, default)
S3method(subhourly2hourly, zoo)
S3method(subhourly2hourly, data.frame)
S3method(subhourly2hourly, matrix)
S3method(subhourly2nminutes, default)
S3method(subhourly2nminutes, zoo)
S3method(subhourly2nminutes, data.frame)
S3method(subhourly2nminutes, matrix)
S3method(weeklyfunction, default)
S3method(weeklyfunction, zoo)
S3method(weeklyfunction, data.frame)
S3method(weeklyfunction, matrix) 


